<?php

/**
  * Defines the settings form for the module.
  */
function commerce_redhen_settings_form() {
  $form = array();

  $form['redhen_membership'] = array(
    '#type' => 'fieldset',
    '#title' => t('Redhen Membership Settings'),
  );

  // Get an array of all Commerce product types.
  $product_types = commerce_product_types();
  $options = array();
  foreach($product_types as $machine_name => $info) {
    $options[$machine_name] = $info['name'];
  }

  $form['redhen_membership']['redhen_product_types'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Product Types'),
    '#description' => t('Enable product types to be used to grant Redhen Memberships.'),
    '#options' => $options,
    '#default_value' => variable_get('commerce_redhen_selected_product_types', array()),
  );

  $form['actions'] = array(
    '#type' => 'container',
  );

  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save configuration')
  );

  return $form;
}

/**
  * Submit handler for the commerce_redhen_settings_form().
  */
function commerce_redhen_settings_form_submit($form, &$form_state) {
  $selected_product_types = array();
  foreach($form_state['values']['redhen_product_types'] as $machine_name => $product_type) {
    // If a product type has been selected, add the relevant fields to it
    if ($product_type) {
      drupal_set_message(t('The <em>@product_type</em> product type now has the require fields to grant memberships.',
        array(
          '@product_type' => $product_type
          )));

      $selected_product_types[] = $product_type;

      /**
        * Add an entityreference field to the product type
        */

      // Create the field if it does not yet exist
      if (!field_info_field('commerce_redhen_membership')) {
        $field = array(
          'field_name' => 'commerce_redhen_membership',
          'type' => 'entityreference',
          'cardinality' => 1,
          'settings' => array(
            'target_type' => 'redhen_membership_type',
          ),
        );

        field_create_field($field);
      }

      // Create the instance if it does not yet exist for this product type
      if (!field_info_instance('commerce_product', 'commerce_redhen_membership', $product_type)) {
        $instance = array(
          'field_name' => 'commerce_redhen_membership',
          'entity_type' => 'commerce_product',
          'label' => 'Redhen Membership Type',
          'bundle' => $product_type,
          'widget' => array(
            'type' => 'options_select',
          ),
        );

        field_create_instance($instance);
      }

      /**
        * Add an interval field to the product type
        */

      // Create the field if it does not yet exist
      if (!field_info_field('commerce_redhen_interval')) {
        $field = array(
          'field_name' => 'commerce_redhen_interval',
          'type' => 'interval',
          'cardinality' => 1,
        );

        field_create_field($field);
      }

      // Create the instance if it does not yet exist for this product type
      if (!field_info_instance('commerce_product', 'commerce_redhen_interval', $product_type)) {
        $instance = array(
          'field_name' => 'commerce_redhen_interval',
          'entity_type' => 'commerce_product',
          'label' => 'Length of Membership',
          'bundle' => $product_type,
        );

        field_create_instance($instance);
      }

    } else {
      // If not selected, then remove the field isntances from the product type
      if ($instance = field_info_instance('commerce_product', 'commerce_redhen_membership', $machine_name)) {
        field_delete_instance($instance);
      }

      if ($instance = field_info_instance('commerce_product', 'commerce_redhen_interval', $machine_name)) {
        field_delete_instance($instance);
      }
    }
  }

  // Save the selection, so we can set the default values in the form
  variable_set('commerce_redhen_selected_product_types', $selected_product_types);
  drupal_set_message(t('The settings have been saved.'));
}
